// - included reference to PhysicalListener  
// - Position, minGain, maxGain, referenceDistance, falloffDistance and
//   falloffFactor included as well as set and get methods
#include <stdio.h>
#include <string.h>
#include <malloc.h>
#include "osc/OscOutboundPacketStream.h"
#include "osc/OscReceivedElements.h"
#include "bergenServer.h"
#include "bergenSound.h"


// The constructor must be given a pointer to a server object; it will tell
// the server object to add this sound to its list of sounds
bergenSound::bergenSound(bergenServer *server)
	{
	server_ = server;
	handle_ = 0;
	isPlaying_ = false;
	if (server)
		server->addSound(this);
	else
		fprintf(stderr,"ERROR: a sound was created with a NULL server pointer; "
			"this may core dump\n");
	}


// The destructor does an automatic kill before the sound object is removed.
bergenSound::~bergenSound(void)
	{
	kill();
	}


// Process a message from the server
void bergenSound::message(osc::ReceivedMessage::const_iterator arg)
	{
	const char* str = arg->AsString();
	if (!strcasecmp(str,"stop"))
		isPlaying_ = false;
	}


// Sets the sound's current amplitude; sends a message to the server program 
// to accomplish this.
void bergenSound::setAmplitude(float amp)
	{
	if (server_)
		{
		char buffer[1024];
		osc::OutboundPacketStream p( buffer, 1024 );
		p << osc::BeginMessage( "object" ) << handle_ << "amplitude" << amp << osc::EndMessage;	
		server_->sendMessage( p );
		}
	}


// Sets the sound's current distance; sends a message to the server program 
// to accomplish this.
void bergenSound::setDistance(float dist)
	{
	if (server_)
		{
		char buffer[1024];
		osc::OutboundPacketStream p( buffer, 1024 );
		p << osc::BeginMessage( "object" ) << handle_ << "distance" << dist << osc::EndMessage;	
		server_->sendMessage( p );
		}
	}


// Sends a message to the server program to start playing the sound.
void bergenSound::play(void)
	{
	if (server_)
		{
		char buffer[1024];
		osc::OutboundPacketStream p( buffer, 1024 );
		p << osc::BeginMessage( "object" ) << handle_ << "play" << osc::EndMessage;	
		server_->sendMessage( p );
		}
	isPlaying_ = true;
	}


// Sends a message to the server program to stop playing the sound. The next
// time a play command is issued, the sound will start again from the
// beginning.
void bergenSound::stop(void)
	{
	if (server_)
		{
		char buffer[1024];
		osc::OutboundPacketStream p( buffer, 1024 );
		p << osc::BeginMessage( "object" ) << handle_ << "stop" << osc::EndMessage;	
		server_->sendMessage( p );
		}
	isPlaying_ = false;
	}


// Sends a message to the server program to pause the sound. The next time a
// play command is issued, the sound will resume from the point at which it
// was paused.
void bergenSound::pause(void)
	{
	if (server_)
		{
		char buffer[1024];
		osc::OutboundPacketStream p( buffer, 1024 );
		p << osc::BeginMessage( "object" ) << handle_ << "pause" << osc::EndMessage;	
		server_->sendMessage( p );
		}
	isPlaying_ = false;
	}


// Sends a message to the server program to remove the sound, and tells the
// server object to remove this sound from its list. The sound object should
// not be used after kill() is called; this is meant to be called from the
// destructor, and should not generally be used directly by applications.
void bergenSound::kill(void)
	{
	killRemote();
	if (server_)
		server_->removeSound(this);
	isPlaying_ = false;
	}


// Sends a message to the server program to remove the sound, and tells the
// server object to remove this sound from its list.
void bergenSound::killRemote(void)
	{
	if (server_)
		{
		char buffer[1024];
		osc::OutboundPacketStream p( buffer, 1024 );
		p << osc::BeginMessage( "object" ) << handle_ << "kill" << osc::EndMessage;	
		server_->sendMessage( p );
		}
	}


// Not implemented
void bergenSound::createRemote(void)
	{
	}


// Sets the sound's position
void bergenSound::setPosition(float x, float y, float z)
	{
	if (server_)
		{
		char buffer[1024];
		osc::OutboundPacketStream p( buffer, 1024 );
		p << osc::BeginMessage( "object" ) << handle_ << "position" << x << y << z << osc::EndMessage;	
		server_->sendMessage( p );
		}
	positionX_ = x;
	positionY_ = y;
	positionZ_ = z;
	}
	

// Sets the sound's attenuation type for distance attenuation
void bergenSound::setAttenuationType(AttenuationType type)
	{
	if (server_)
		{
		char buffer[1024];
		osc::OutboundPacketStream p( buffer, 1024 );
		p << osc::BeginMessage( "object" ) << handle_ << "attenuation" << (int)type << osc::EndMessage;	
		server_->sendMessage( p );
		}
	attenuationType_=type;	
	}


// Gets the sound's attenuation type 
AttenuationType bergenSound::attenuationType(void)
	{
	return attenuationType_;
	}


// Sets the sound's min gain 
void bergenSound::setMinGain(float gain)
	{
	if (server_)
		{
		char buffer[1024];
		osc::OutboundPacketStream p( buffer, 1024 );
		p << osc::BeginMessage( "object" ) << handle_ << "minGain" << gain << osc::EndMessage;	
		server_->sendMessage( p );
		}
	minGain_=gain;	
	}


// Gets the sound's minGain
float bergenSound::minGain(void)
	{
	return minGain_;
	}


// Sets the sound's max gain 
void bergenSound::setMaxGain(float gain)
	{
	if (server_)
		{
		char buffer[1024];
		osc::OutboundPacketStream p( buffer, 1024 );
		p << osc::BeginMessage( "object" ) << handle_ << "maxGain" << gain << osc::EndMessage;	
		server_->sendMessage( p );
		}
	maxGain_=gain;	
	}


// Gets the sound's maxGain
float bergenSound::maxGain(void)
	{
	return maxGain_;
	}


// Sets the sound's reference distance
void bergenSound::setReferenceDistance(float distance)
	{
	if (server_)
		{
		char buffer[1024];
		osc::OutboundPacketStream p( buffer, 1024 );
		p << osc::BeginMessage( "object" ) << handle_ << "referenceDistance" << distance << osc::EndMessage;	
		server_->sendMessage( p );
		}
	referenceDistance_=distance;	
	}


// Gets the sound's referenceDistance Value
float bergenSound::referenceDistance(void)
	{
	return referenceDistance_;
	}


// Sets the sound's falloff distance
void bergenSound::setFalloffDistance(float distance)
	{
	if (server_)
		{
		char buffer[1024];
		osc::OutboundPacketStream p( buffer, 1024 );
		p << osc::BeginMessage( "object" ) << handle_ << "falloffDistance" << distance << osc::EndMessage;	
		server_->sendMessage( p );
		}
	falloffDistance_=distance;	
	}


// Gets the sound's falloffDitance
float bergenSound::falloffDistance(void)
	{
	return falloffDistance_;
	}


// Sets the sound's falloff factor
void bergenSound::setFalloffFactor(float factor)
	{
	if (server_)
		{
		char buffer[1024];
		osc::OutboundPacketStream p( buffer, 1024 );
		p << osc::BeginMessage( "object" ) << handle_ << "falloffFactor" << factor << osc::EndMessage;	
		server_->sendMessage( p );
		}
	falloffFactor_=factor;	
	}


// Gets the sound's falloffFactor
float bergenSound::falloffFactor(void)
	{
	return falloffFactor_;
	}
